#' ---
#' title: "Make predictions for all mRNA isoform pairs"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

import sys
import csv
import time
import collections
import numpy as np
import pandas as pd
import sklearn.metrics
from sklearn.ensemble import RandomForestClassifier
from sklearn.externals import joblib

# Define the makehash function to create a multi-level dictionary to store all prediction metrics.
def makehash():
    return collections.defaultdict(makehash)

measures = makehash()

# Define a list of numbers from 00 - 24 to be used when reading files to predict.
a=["%02d" % x for x in range(25)]

# Define a dictionary to store the models.
model = {}

# Read the saved models
_, _, model, _, _ = joblib.load("../Data/MMU_modelJoblib_13Nov18.pkl")

# Read the training file, replace all NA values with 0 and define the labels.
# For each file containing a subset of ~2.9billion mRNA Isoform pairs, define a pandas dataframe to store the predictions for every tissue, make predictions for every tissue and save the predictions.
for test, label in zip(a, a):
    testing = pd.read_csv("../MMU_Predict_%s.gz" % test ,sep=',', header=None, low_memory=True)
    testing.fillna(value=0, inplace=True)
    print("File %s read" % test, flush=True)
    predictions = pd.DataFrame({'Iso1':testing.iloc[:, 0],'Iso2':testing.iloc[:, 1]})
    start_time = time.time()
    for col1, col2, colnames in zip([np.r_[3:12, 13:30], np.r_[3:13,14:30], np.r_[3:14,15:30], np.r_[3:15,16:30], np.r_[3:16,17:30], np.r_[3:17,18:30], np.r_[3:18,19:30], np.r_[3:19,20:30], np.r_[3:20,21:30], np.r_[3:21,22:30], np.r_[3:22,23:30], np.r_[3:23,24:30], np.r_[3:24,25:30], np.r_[3:25,26:30], np.r_[3:26,27:30], np.r_[3:27,28:30], np.r_[3:28,29], np.r_[3:30]], [np.r_[2:11, 12:29], np.r_[2:12,13:29], np.r_[2:13,14:29], np.r_[2:14,15:29], np.r_[2:15,16:29], np.r_[2:16,17:29], np.r_[2:17,18:29], np.r_[2:18,19:29], np.r_[2:19,20:29], np.r_[2:20,21:29], np.r_[2:21,22:29], np.r_[2:22,23:29], np.r_[2:23,24:29], np.r_[2:24,25:29], np.r_[2:25,26:29], np.r_[2:26,27:29], np.r_[2:27,28], np.r_[2:29]],['withoutHeart', 'withoutLiver', 'withoutKidney', 'withoutAdGland', 'withoutForebrain', 'withoutHindbrain', 'withoutEmbFacPro', 'withoutLintestine', 'withoutLung', 'withoutLimb', 'withoutMidbrain', 'withoutNtube', 'withoutSintestine', 'withoutOvary', 'withoutSpleen', 'withoutStomach', 'withoutThymus', 'All']):
        start = time.time()
        testArr = testing.iloc[:, col2]
        results = model[colnames].predict(testArr)
        probs = model[colnames].predict_proba(testArr)
        predictions['pred_%s' % colnames] = results
        predictions['prob_%s' % colnames] = probs[:, 1]
        end = time.time()
        print('Predictions for %(x)s in %(z)s done in: %(y)s seconds' % {"x" : colnames, "y" : (end-start), "z" : test}, flush=True)
    predictions.to_csv('Predictions_MMU_%s.csv' % test, index=False)
    end_time = time.time()
    print('Predictions for %(x)s done in: %(y)s seconds' % {"x" : test, "y" : (end_time - start_time)}, flush=True)

