#' ---
#' title: "Perform validation on the validation dataset"
#' authors: "Gaurav Kandoi, Julie A. Dickerson"
#' date: "November 13, 2018"
#' affiliation: "Iowa State University of Science and Technology"
#' license: "CC BY 4.0"
#' ---

import numpy as np
import pandas as pd
import sklearn.metrics
import matplotlib as mpl
from sklearn.metrics import roc_curve, roc_auc_score
from sklearn.metrics import precision_recall_curve, average_precision_score

mpl.use('agg')

import matplotlib.pyplot as plt
from pylab import *

# Read the predictions for the validation dataset.
Predictions = pd.read_csv("../Data/validatedTPPairsLabelled13Nov18.txt.gz",   sep = ",", header = 0)

# Calculate the precision, recall, average precision, true positive rate, false positive rate and area under the receiver operating characteristics curve.
precision, recall, _ = precision_recall_curve(Predictions['V3'], Predictions['prob_All'])
average_precision = average_precision_score(Predictions['V3'], Predictions['prob_All'])
fpr, tpr, _ = roc_curve(Predictions['V3'], Predictions['prob_All'], drop_intermediate=False)
roc_auc = roc_auc_score(Predictions['V3'], Predictions['prob_All'])

# Plot the Precision-Recall and Receiver Operating Characteristics curve for the validation dataset.
fig = plt.figure(figsize=(10, 10), dpi = 80)
plt.subplot(2,1,1)
plt.step(recall, precision, color='brown', label='PR curve (area = %0.3f)' % average_precision)
plt.xlim([0.0, 1.0])
plt.ylim([0.0, 1.0])
plt.xlabel('Recall', fontsize = 17)
plt.ylabel('Precision', fontsize = 17)
plt.title('Precision-Recall curve for validation set', fontsize = 17)
plt.legend(loc='lower right', fontsize = 12)

plt.subplot(2,1,2)
plt.plot(fpr, tpr, color='brown', label='ROC curve (area = %0.3f)' % roc_auc)
plt.plot([0, 1], [0, 1], linestyle='--', color='black', label='Random')
plt.xlim([0.0, 1.0])
plt.ylim([0.0, 1.0])
plt.xlabel('False Positive Rate', fontsize = 17)
plt.ylabel('True Positive Rate', fontsize = 17)
plt.title('Receiver Operating Characteristic curve for validation set', fontsize = 17)
plt.legend(loc='lower right', fontsize = 12)
# Save the plot.
plt.tight_layout()
fig.savefig('../Data/Fig7.tiff')
